@@ -1,29 +1,29 @@
-/* Time-stamp: <2004-01-12 17:27:27 jcs>
+/* Time-stamp: <2004-12-30 13:32:09 jcs>
 |
 |  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
 |  Part of the gtkpod project.
-| 
+|
 |  URL: http://gtkpod.sourceforge.net/
-| 
+|
 |  This program is free software; you can redistribute it and/or modify
 |  it under the terms of the GNU General Public License as published by
 |  the Free Software Foundation; either version 2 of the License, or
 |  (at your option) any later version.
-| 
+|
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 |  GNU General Public License for more details.
-| 
+|
 |  You should have received a copy of the GNU General Public License
 |  along with this program; if not, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-| 
+|
 |  iTunes and iPod are trademarks of Apple
-| 
+|
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_songs.c,v 1.35 2004/01/17 09:37:48 jcsjcs Exp $
+|  $Id: display_songs.c,v 1.54 2004/12/30 04:33:58 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -39,7 +39,7 @@
 #include "support.h"
 #include "prefs.h"
 #include "display_private.h"
-#include "song.h"
+#include "track.h"
 #include "playlist.h"
 #include "callbacks.h"
 #include "misc.h"
@@ -71,29 +71,75 @@ static GtkTargetEntry tm_drop_types [] = {
 };
 
 
+/* Note: the toggle buttons for tag_autoset and display_col in the
+ * prefs_window are named after the the TM_COLUM_* numbers defined in
+ * display.h (Title: tag_autoset0, Artist: tag_autoset1 etc.). Since
+ * the labels to the buttons are set in prefs_window.c when creating
+ * the window, you only need to name the buttons in the intended order
+ * using glade-2. There is no need to label them. */
 /* Strings associated to the column headers */
 const gchar *tm_col_strings[] = {
-    N_("Title"),
+    N_("Title"),             /*  0 */
     N_("Artist"),
     N_("Album"),
     N_("Genre"),
     N_("Composer"),
-    N_("Track Nr (#)"),
+    N_("Track Nr (#)"),      /*  5 */
     N_("iPod ID"),
     N_("PC File"),
     N_("Transferred"),
     N_("File Size"),
-    N_("Play Time"),
+    N_("Play Time"),         /* 10 */
     N_("Bitrate"),
     N_("Playcount"),
     N_("Rating"),
     N_("Time played"),
-    N_("Time modified"),
+    N_("Time modified"),     /* 15 */
     N_("Volume"),
     N_("Year"),
     N_("CD Nr"),
+    N_("Time created"),
+    N_("iPod File"),         /* 20 */
+    N_("Soundcheck"),
+    N_("Samplerate"),
+    N_("BPM"),
+    N_("Kind"),
+    N_("Grouping"),          /* 25 */
+    N_("Compilation"),
+    NULL };
+/* Tooltips for prefs window */
+const gchar *tm_col_tooltips[] = {
+    NULL,                                              /*  0 */
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    N_("Track Nr. and total number of tracks on CD"),  /*  5 */
+    NULL,
+    N_("Name of file on PC, if available"),
+    N_("Whether the file has already been "
+       "transferred to the iPod or not"),
+    NULL,
+    NULL,                                              /* 10 */
+    NULL,
+    N_("Number of times the track has been played"),
+    N_("Star rating from 0 to 5"),
+    N_("Time track has last been played"),
+    N_("Time track has last been modified"),      /* 15 */
+    N_("Manual volume adjust"),
+    NULL,
+    N_("CD Nr. and total number of CDS in set"),
+    N_("Time track has been created (timestamp of file)"),
+    N_("Name of file on the iPod"),                    /* 20 */
+    N_("Volume adjust in dB (replay gain) -- "
+       "you need to activate 'soundcheck' on the iPod"),
+    NULL,
+    N_("Supposedly something that tells the iPod to "
+       "increase or decrease the playback speed"),
+    NULL,
+    NULL,                  /* 25 */
+    NULL,
     NULL };
-
 
 
 /* ---------------------------------------------------------------- */
@@ -155,33 +201,19 @@ void tm_remove_track (Track *track)
 
 
 /* Remove all tracks from the display model */
-/* ATTENTION: the treeview and model might be changed by calling this
-   function */
-void tm_remove_all_tracks (gboolean clear_sort)
+void tm_remove_all_tracks ()
 {
   GtkTreeModel *model = gtk_tree_view_get_model (track_treeview);
   GtkTreeIter iter;
-  gint column;
-  GtkSortType order;
 
   while (gtk_tree_model_get_iter_first (model, &iter))
   {
       gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
   }
-  if(clear_sort && (prefs_get_tm_sort () == SORT_NONE) &&
-     gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE (model),
-					   &column, &order))
-  { /* recreate track treeview to unset sorted column */
-      if (column >= 0)
-      {
-	  tm_store_col_order ();
-	  display_update_default_sizes ();
-	  tm_create_treeview ();
-      }
-  }
+  tm_store_col_order ();
+  tm_update_default_sizes ();
 }
 
-
 /* find out at which position column @tm_item is displayed */
 /* static gint tm_get_col_position (TM_item tm_item) */
 /* { */
@@ -273,7 +305,7 @@ GList *gtk_tree_selection_get_selected_rows (GtkTreeSelection *selection,
 
     gtk_tree_selection_selected_foreach (selection, gtssf, &gts);
     return list;
-} 
+}
 #endif
 
 
@@ -315,9 +347,9 @@ static const gchar *display_get_disk_string (Track *track)
 }
 
 
-/* Called when editable cell is being edited. Stores new data to
-   the track list. Eventually the ID3 tags in the corresponding
-   files should be changed as well, if activated in the pref settings */
+/* Called when editable cell is being edited. Stores new data to the
+   track list. ID3 tags in the corresponding files are updated as
+   well, if activated in the pref settings */
 static void
 tm_cell_edited (GtkCellRendererText *renderer,
 		const gchar         *path_string,
@@ -325,20 +357,21 @@ tm_cell_edited (GtkCellRendererText *renderer,
 		gpointer             data)
 {
   GtkTreeModel *model;
-  GtkTreeSelection *selection; 
-  TM_item column; 
+  GtkTreeSelection *selection;
+  TM_item column;
   gboolean multi_edit;
   gint sel_rows_num;
-  GList *row_list, *row_node, *first; 
+  GList *row_list, *row_node, *first;
+  time_t t;
 
 
   column = (TM_item) g_object_get_data(G_OBJECT(renderer), "column");
   multi_edit = prefs_get_multi_edit ();
   if (column == TM_COLUMN_TITLE)
       multi_edit &= prefs_get_multi_edit_title ();
-  selection = gtk_tree_view_get_selection(track_treeview); 
-  row_list = gtk_tree_selection_get_selected_rows(selection, &model); 
-  
+  selection = gtk_tree_view_get_selection(track_treeview);
+  row_list = gtk_tree_selection_get_selected_rows(selection, &model);
+
 /*   printf("tm_cell_edited: column: %d\n", column); */
 
   sel_rows_num = g_list_length (row_list);
@@ -348,28 +381,30 @@ tm_cell_edited (GtkCellRendererText *renderer,
 
   first = g_list_first (row_list);
 
-  for (row_node = first; 
-       row_node && (multi_edit || (row_node == first)); 
+  for (row_node = first;
+       row_node && (multi_edit || (row_node == first));
        row_node = g_list_next(row_node))
   {
-     Track *track; 
-     gboolean changed;        
-     GtkTreeIter iter; 
+     Track *track;
+     gboolean changed;
+     GtkTreeIter iter;
      gint32 nr;
-     gchar **itemp_utf8, *str; 
-     gunichar2 **itemp_utf16; 
+     gchar **itemp_utf8, *str;
+     gunichar2 **itemp_utf16;
 
-     gtk_tree_model_get_iter(model, &iter, (GtkTreePath *) row_node->data); 
-     gtk_tree_model_get(model, &iter, READOUT_COL, &track, -1); 
-     changed = FALSE; 
+     gtk_tree_model_get_iter(model, &iter, (GtkTreePath *) row_node->data);
+     gtk_tree_model_get(model, &iter, READOUT_COL, &track, -1);
+     changed = FALSE;
 
-     switch(column) 
+     switch(column)
      {
      case TM_COLUMN_TITLE:
      case TM_COLUMN_ALBUM:
      case TM_COLUMN_ARTIST:
      case TM_COLUMN_GENRE:
      case TM_COLUMN_COMPOSER:
+     case TM_COLUMN_FDESC:
+     case TM_COLUMN_GROUPING:
         itemp_utf8 = track_get_item_pointer_utf8 (track, TM_to_T (column));
         itemp_utf16 = track_get_item_pointer_utf16 (track, TM_to_T (column));
         if (g_utf8_collate (*itemp_utf8, new_text) != 0)
@@ -378,7 +413,7 @@ tm_cell_edited (GtkCellRendererText *renderer,
            g_free (*itemp_utf16);
            *itemp_utf8 = g_strdup (new_text);
            *itemp_utf16 = g_utf8_to_utf16 (new_text, -1, NULL, NULL, NULL);
-           changed = TRUE; 
+           changed = TRUE;
         }
         break;
      case TM_COLUMN_TRACK_NR:
@@ -386,7 +421,7 @@ tm_cell_edited (GtkCellRendererText *renderer,
         if ((nr >= 0) && (nr != track->track_nr))
         {
            track->track_nr = nr;
-           changed = TRUE; 
+           changed = TRUE;
         }
 	str = strrchr (new_text, '/');
 	if (str)
@@ -395,7 +430,7 @@ tm_cell_edited (GtkCellRendererText *renderer,
 	    if ((nr >= 0) && (nr != track->tracks))
 	    {
 		track->tracks = nr;
-		changed = TRUE; 
+		changed = TRUE;
 	    }
 	}
 	g_object_set (G_OBJECT (renderer),
@@ -406,7 +441,7 @@ tm_cell_edited (GtkCellRendererText *renderer,
         if ((nr >= 0) && (nr != track->cd_nr))
         {
            track->cd_nr = nr;
-           changed = TRUE; 
+           changed = TRUE;
         }
 	str = strrchr (new_text, '/');
 	if (str)
@@ -415,7 +450,7 @@ tm_cell_edited (GtkCellRendererText *renderer,
 	    if ((nr >= 0) && (nr != track->cds))
 	    {
 		track->cds = nr;
-		changed = TRUE; 
+		changed = TRUE;
 	    }
 	}
 	g_object_set (G_OBJECT (renderer),
@@ -428,7 +463,7 @@ tm_cell_edited (GtkCellRendererText *renderer,
 	   g_free (track->year_str);
 	   track->year_str = g_strdup_printf ("%d", nr);
            track->year = nr;
-           changed = TRUE; 
+           changed = TRUE;
         }
         break;
      case TM_COLUMN_PLAYCOUNT:
@@ -436,7 +471,7 @@ tm_cell_edited (GtkCellRendererText *renderer,
         if ((nr >= 0) && (nr != track->playcount))
         {
            track->playcount = nr;
-           changed = TRUE; 
+           changed = TRUE;
         }
         break;
      case TM_COLUMN_RATING:
@@ -447,17 +482,57 @@ tm_cell_edited (GtkCellRendererText *renderer,
            changed = TRUE;
         }
         break;
+     case TM_COLUMN_TIME_CREATED:
      case TM_COLUMN_TIME_PLAYED:
      case TM_COLUMN_TIME_MODIFIED:
-        break;
+	 t = time_string_to_time (new_text);
+	 if ((t != -1) && (t != time_get_time (track, column)))
+	 {
+	     time_set_time (track, t, column);
+	     changed = TRUE;
+	 }
+	 break;
      case TM_COLUMN_VOLUME:
         nr = atoi (new_text);
-        if ((nr <= 100) && (nr >= -100) && (nr != track->volume))
+        if (nr != track->volume)
         {
 	    track->volume = nr;
 	    changed = TRUE;
         }
         break;
+     case TM_COLUMN_SOUNDCHECK:
+	nr = replaygain_to_soundcheck (atof (new_text));
+/* 	printf("%d : %f\n", nr, atof (new_text)); */
+        if (nr != track->soundcheck)
+        {
+	    track->soundcheck = nr;
+	    changed = TRUE;
+        }
+        break;
+     case TM_COLUMN_BITRATE:
+        nr = atoi (new_text);
+        if (nr != track->bitrate)
+        {
+	    track->bitrate = nr;
+	    changed = TRUE;
+        }
+        break;
+     case TM_COLUMN_SAMPLERATE:
+        nr = atoi (new_text);
+        if (nr != track->samplerate)
+        {
+	    track->samplerate = nr;
+	    changed = TRUE;
+        }
+        break;
+     case TM_COLUMN_BPM:
+        nr = atoi (new_text);
+        if (nr != track->BPM)
+        {
+	    track->BPM = nr;
+	    changed = TRUE;
+        }
+        break;
      default:
         g_warning ("Programming error: tm_cell_edited: unknown track cell (%d) edited\n", column);
         break;
@@ -469,8 +544,8 @@ tm_cell_edited (GtkCellRendererText *renderer,
         pm_track_changed (track); /* notify playlist model... */
         data_changed ();        /* indicate that data has changed */
 
-        if (prefs_get_id3_write()) 
-        { 
+        if (prefs_get_id3_write())
+        {
 	    /* T_item tag_id;*/
            /* should we update all ID3 tags or just the one
               changed? -- obsoleted in 0.71*/
@@ -486,8 +561,8 @@ tm_cell_edited (GtkCellRendererText *renderer,
 
   if (multi_edit && (sel_rows_num > 1)) release_widgets ();
 
-  g_list_foreach(row_list, (GFunc) gtk_tree_path_free, NULL); 
-  g_list_free(row_list); 
+  g_list_foreach(row_list, (GFunc) gtk_tree_path_free, NULL);
+  g_list_free(row_list);
 }
 
 
@@ -518,9 +593,11 @@ static void tm_cell_data_func (GtkTreeViewColumn *tree_column,
   case TM_COLUMN_ALBUM:
   case TM_COLUMN_GENRE:
   case TM_COLUMN_COMPOSER:
+  case TM_COLUMN_FDESC:
+  case TM_COLUMN_GROUPING:
       item_utf8 = track_get_item_utf8 (track, TM_to_T (column));
       g_object_set (G_OBJECT (renderer),
-		    "text", item_utf8, 
+		    "text", item_utf8,
 		    "editable", TRUE, NULL);
       break;
   case TM_COLUMN_TRACK_NR:
@@ -542,7 +619,7 @@ static void tm_cell_data_func (GtkTreeViewColumn *tree_column,
 	  g_object_set (G_OBJECT (renderer),
 			"text", text,
 			"xalign", 1.0, NULL);
-      } 
+      }
       else
       {
 	  g_object_set (G_OBJECT (renderer),
@@ -553,9 +630,17 @@ static void tm_cell_data_func (GtkTreeViewColumn *tree_column,
   case TM_COLUMN_PC_PATH:
       g_object_set (G_OBJECT (renderer), "text", track->pc_path_utf8, NULL);
       break;
+  case TM_COLUMN_IPOD_PATH:
+      g_object_set (G_OBJECT (renderer), "text", track->ipod_path, NULL);
+      break;
   case TM_COLUMN_TRANSFERRED:
       g_object_set (G_OBJECT (renderer), "active", track->transferred, NULL);
       break;
+  case TM_COLUMN_COMPILATION:
+      g_object_set (G_OBJECT (renderer),
+		    "active", !track->compilation,
+		    "activatable", TRUE, NULL);
+      break;
   case TM_COLUMN_SIZE:
       snprintf (text, 20, "%d", track->size);
       g_object_set (G_OBJECT (renderer),
@@ -573,6 +658,21 @@ static void tm_cell_data_func (GtkTreeViewColumn *tree_column,
       snprintf (text, 20, "%dk", track->bitrate);
       g_object_set (G_OBJECT (renderer),
 		    "text", text,
+		    "editable", TRUE,
+		    "xalign", 1.0, NULL);
+      break;
+  case TM_COLUMN_SAMPLERATE:
+      snprintf (text, 20, "%d", track->samplerate);
+      g_object_set (G_OBJECT (renderer),
+		    "text", text,
+		    "editable", TRUE,
+		    "xalign", 1.0, NULL);
+      break;
+  case TM_COLUMN_BPM:
+      snprintf (text, 20, "%d", track->BPM);
+      g_object_set (G_OBJECT (renderer),
+		    "text", text,
+		    "editable", TRUE,
 		    "xalign", 1.0, NULL);
       break;
   case TM_COLUMN_PLAYCOUNT:
@@ -598,11 +698,11 @@ static void tm_cell_data_func (GtkTreeViewColumn *tree_column,
       break;
   case TM_COLUMN_TIME_PLAYED:
   case TM_COLUMN_TIME_MODIFIED:
+  case TM_COLUMN_TIME_CREATED:
       buf = time_field_to_string (track, column);
       g_object_set (G_OBJECT (renderer),
 		    "text", buf,
-/* don't know how to make it editable yet */
-/* 		    "editable", TRUE, */
+ 		    "editable", TRUE,
 		    "xalign", 0.0, NULL);
       C_FREE (buf);
       break;
@@ -613,12 +713,138 @@ static void tm_cell_data_func (GtkTreeViewColumn *tree_column,
 		    "editable", TRUE,
 		    "xalign", 1.0, NULL);
       break;
+  case TM_COLUMN_SOUNDCHECK:
+/*       printf ("%p:%f : %d\n", track, */
+/* 	      soundcheck_to_replaygain (track->soundcheck), */
+/* 	      track->soundcheck); */
+      snprintf (text, 20, "%0.2f",
+		soundcheck_to_replaygain (track->soundcheck));
+      g_object_set (G_OBJECT (renderer),
+		    "text", text,
+		    "editable", TRUE,
+		    "xalign", 1.0, NULL);
+      break;
   default:
       g_warning ("Programming error: unknown column in tm_cell_data_func: %d\n", column);
       break;
   }
 }
 
+
+
+/* Called when a toggle cell is being changed. Stores new data to the
+   track list. */
+static void
+tm_cell_toggled (GtkCellRendererToggle *renderer,
+		 gchar *arg1,
+		 gpointer user_data)
+{
+  GtkTreeModel *model;
+  GtkTreeSelection *selection;
+  TM_item column;
+  gboolean multi_edit;
+  gint sel_rows_num;
+  GList *row_list, *row_node, *first;
+  gboolean active;
+
+  column = (TM_item) g_object_get_data(G_OBJECT(renderer), "column");
+  multi_edit = prefs_get_multi_edit ();
+  selection = gtk_tree_view_get_selection(track_treeview);
+  row_list = gtk_tree_selection_get_selected_rows(selection, &model);
+
+/*  printf("tm_cell_toggled: column: %d, arg1: %p\n", column, arg1); */
+
+  sel_rows_num = g_list_length (row_list);
+
+  /* block widgets and update display if multi-edit is active */
+  if (multi_edit && (sel_rows_num > 1)) block_widgets ();
+
+  first = g_list_first (row_list);
+
+
+  g_object_get (G_OBJECT (renderer), "active", &active, NULL);
+
+  for (row_node = first;
+       row_node && (multi_edit || (row_node == first));
+       row_node = g_list_next(row_node))
+  {
+     Track *track;
+     gboolean changed;
+     GtkTreeIter iter;
+
+     gtk_tree_model_get_iter(model, &iter, (GtkTreePath *) row_node->data);
+     gtk_tree_model_get(model, &iter, READOUT_COL, &track, -1);
+     changed = FALSE;
+
+     switch(column)
+     {
+     case TM_COLUMN_TITLE:
+	 if ((active && (track->checked == 0)) ||
+	     (!active && (track->checked == 1)))
+	     changed = TRUE;
+	 if (active) track->checked = 1;
+	 else        track->checked = 0;
+	 break;
+     case TM_COLUMN_COMPILATION:
+	 if ((active && (track->compilation == 0)) ||
+	     (!active && (track->compilation == 1)))
+	     changed = TRUE;
+	 if (active) track->compilation = 1;
+	 else        track->compilation = 0;
+        break;
+     default:
+        g_warning ("Programming error: tm_cell_toggled: unknown track cell (%d) edited\n", column);
+        break;
+     }
+/*      printf ("  changed: %d\n", changed); */
+     if (changed)
+     {
+        track->time_modified = itunesdb_time_get_mac_time ();
+/*        pm_track_changed (track);  notify playlist model... -- not
+ *        necessary here because only the track model is affected */
+        data_changed ();        /* indicate that data has changed */
+     }
+     while (widgets_blocked && gtk_events_pending ())  gtk_main_iteration ();
+  }
+
+  if (multi_edit && (sel_rows_num > 1)) release_widgets ();
+
+  g_list_foreach(row_list, (GFunc) gtk_tree_path_free, NULL);
+  g_list_free(row_list);
+}
+
+
+
+/* The track data is stored in a separate list (static GList *tracks)
+   and only pointers to the corresponding Track structure are placed
+   into the model.
+   This function reads the data for the given cell from the list and
+   passes it to the renderer. */
+static void tm_cell_data_func_toggle (GtkTreeViewColumn *tree_column,
+				      GtkCellRenderer   *renderer,
+				      GtkTreeModel      *model,
+				      GtkTreeIter       *iter,
+				      gpointer           data)
+{
+  Track *track;
+  TM_item column;
+
+  column = (TM_item)g_object_get_data (G_OBJECT (renderer), "column");
+  gtk_tree_model_get (model, iter, READOUT_COL, &track, -1);
+
+  switch (column)
+  {
+  case TM_COLUMN_TITLE:
+      g_object_set (G_OBJECT (renderer),
+		    "active", !track->checked,
+		    "activatable", TRUE, NULL);
+      break;
+  default:
+      g_warning ("Programming error: unknown column in tm_cell_data_func_toggle: %d\n", column);
+      break;
+  }
+}
+
 /**
  * tm_get_nr_of_tracks - get the number of tracks displayed
  * currently in the track model Returns - the number of tracks displayed
@@ -631,7 +857,7 @@ tm_get_nr_of_tracks(void)
     guint result = 0;
     gboolean valid = FALSE;
     GtkTreeModel *tm = NULL;
-			    
+
     if((tm = gtk_tree_view_get_model(GTK_TREE_VIEW(track_treeview))))
     {
 	if((valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(tm),&i)))
@@ -662,7 +888,7 @@ tm_rows_reordered (void)
 {
     Playlist *current_pl = pm_get_selected_playlist();
     gboolean changed = FALSE;
-		    
+
     if(current_pl)
     {
 	GtkTreeModel *tm = NULL;
@@ -680,7 +906,7 @@ tm_rows_reordered (void)
 		Track *new_track;
 		gint old_position;
 
-		gtk_tree_model_get(tm, &i, READOUT_COL, &new_track, -1); 
+		gtk_tree_model_get(tm, &i, READOUT_COL, &new_track, -1);
 		new_list = g_list_append(new_list, new_track);
 		/* what position was this track in before? */
 		old_position = g_list_index (current_pl->members, new_track);
@@ -743,7 +969,7 @@ tm_rows_reordered (void)
 
 
 static void
-on_trackids_list_foreach ( GtkTreeModel *tm, GtkTreePath *tp, 
+on_trackids_list_foreach ( GtkTreeModel *tm, GtkTreePath *tp,
 			  GtkTreeIter *i, gpointer data)
 {
     Track *s = NULL;
@@ -773,7 +999,7 @@ tm_get_selected_trackids(void)
 }
 
 static gboolean
-on_all_trackids_list_foreach (GtkTreeModel *tm, GtkTreePath *tp, 
+on_all_trackids_list_foreach (GtkTreeModel *tm, GtkTreePath *tp,
 			     GtkTreeIter *i, gpointer data)
 {
     on_trackids_list_foreach (tm, tp, i, data);
@@ -797,7 +1023,7 @@ tm_get_all_trackids(void)
 }
 
 static void
-on_tracks_list_foreach ( GtkTreeModel *tm, GtkTreePath *tp, 
+on_tracks_list_foreach ( GtkTreeModel *tm, GtkTreePath *tp,
 			GtkTreeIter *i, gpointer data)
 {
     Track *s = NULL;
@@ -828,7 +1054,7 @@ tm_get_selected_tracks(void)
 
 
 static gboolean
-on_all_tracks_list_foreach (GtkTreeModel *tm, GtkTreePath *tp, 
+on_all_tracks_list_foreach (GtkTreeModel *tm, GtkTreePath *tp,
 			   GtkTreeIter *i, gpointer data)
 {
     on_tracks_list_foreach (tm, tp, i, data);
@@ -869,7 +1095,7 @@ void tm_stop_editing (gboolean cancel)
 	   seem to work the way intended) */
 	gboolean me = prefs_get_multi_edit ();
 	prefs_set_multi_edit (FALSE);
-	if (!cancel && col->editable_widget)  
+	if (!cancel && col->editable_widget)
 	    gtk_cell_editable_editing_done (col->editable_widget);
 	if (col->editable_widget)
 	    gtk_cell_editable_remove_widget (col->editable_widget);
@@ -878,34 +1104,27 @@ void tm_stop_editing (gboolean cancel)
 }
 
 
-/* Function used to compare two cells during sorting (track view) */
-gint tm_data_compare_func (GtkTreeModel *model,
-			GtkTreeIter *b,
-			GtkTreeIter *a,
-			gpointer user_data)
+
+/* Function to compare @tm_item of @track1 and @track2. Used by
+   tm_data_compare_func() */
+static gint tm_data_compare (Track *track1, Track *track2,
+			     TM_item tm_item)
 {
-  Track *track1;
-  Track *track2;
-  gint column;
-  TM_item tm_item;
-  GtkSortType order;
   gint cmp;
 
-  gtk_tree_model_get (model, a, READOUT_COL, &track1, -1);
-  gtk_tree_model_get (model, b, READOUT_COL, &track2, -1);
-  if(gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE (model),
-					   &column, &order) == FALSE) return 0;
-  tm_item = (TM_item) column;
-  /*printf ("tm_comp: %d\n", tm_item);*/
   switch (tm_item)
   {
   case TM_COLUMN_TITLE:
-  case TM_COLUMN_ARTIST:
   case TM_COLUMN_ALBUM:
   case TM_COLUMN_GENRE:
   case TM_COLUMN_COMPOSER:
+  case TM_COLUMN_FDESC:
+  case TM_COLUMN_GROUPING:
       return compare_string (track_get_item_utf8 (track1, TM_to_T (tm_item)),
 			     track_get_item_utf8 (track2, TM_to_T (tm_item)));
+  case TM_COLUMN_ARTIST:
+      return compare_string_fuzzy (track_get_item_utf8 (track1, TM_to_T (tm_item)),
+			     track_get_item_utf8 (track2, TM_to_T (tm_item)));
   case TM_COLUMN_TRACK_NR:
       cmp = track1->tracks - track2->tracks;
       if (cmp == 0) cmp = track1->track_nr - track2->track_nr;
@@ -918,44 +1137,72 @@ gint tm_data_compare_func (GtkTreeModel *model,
       return track1->ipod_id - track2->ipod_id;
   case TM_COLUMN_PC_PATH:
       return g_utf8_collate (track1->pc_path_utf8, track2->pc_path_utf8);
+  case TM_COLUMN_IPOD_PATH:
+      return g_utf8_collate (track1->ipod_path, track2->ipod_path);
   case TM_COLUMN_TRANSFERRED:
       if(track1->transferred == track2->transferred) return 0;
       if(track1->transferred == TRUE) return 1;
       else return -1;
+  case TM_COLUMN_COMPILATION:
+      if(track1->compilation == track2->compilation) return 0;
+      if(track1->compilation == TRUE) return 1;
+      else return -1;
   case TM_COLUMN_SIZE:
       return track1->size - track2->size;
   case TM_COLUMN_TRACKLEN:
       return track1->tracklen - track2->tracklen;
   case TM_COLUMN_BITRATE:
       return track1->bitrate - track2->bitrate;
+  case TM_COLUMN_SAMPLERATE:
+      return track1->samplerate - track2->samplerate;
+  case TM_COLUMN_BPM:
+      return track1->BPM - track2->BPM;
   case TM_COLUMN_PLAYCOUNT:
       return track1->playcount - track2->playcount;
   case  TM_COLUMN_RATING:
       return track1->rating - track2->rating;
+  case TM_COLUMN_TIME_CREATED:
   case TM_COLUMN_TIME_PLAYED:
   case TM_COLUMN_TIME_MODIFIED:
       return COMP (time_get_time (track1, tm_item),
 		   time_get_time (track2, tm_item));
   case  TM_COLUMN_VOLUME:
       return track1->volume - track2->volume;
+  case  TM_COLUMN_SOUNDCHECK:
+      /* If soundcheck is unset (0) use 0 dB (1000) */
+      return (track1->soundcheck? track1->soundcheck:1000) - 
+	  (track2->soundcheck? track2->soundcheck:1000);
   case TM_COLUMN_YEAR:
       return track1->year - track2->year;
   default:
-      g_warning ("Programming error: tm_data_compare_func: no sort method for column %d\n", column);
+      g_warning ("Programming error: tm_data_compare_func: no sort method for tm_item %d\n", tm_item);
       break;
   }
   return 0;
 }
 
 
-gint default_comp  (GtkTreeModel *model,
-		    GtkTreeIter *a,
-		    GtkTreeIter *b,
-		    gpointer user_data)
+/* Function used to compare two cells during sorting (track view) */
+gint tm_data_compare_func (GtkTreeModel *model,
+			GtkTreeIter *a,
+			GtkTreeIter *b,
+			gpointer user_data)
 {
-    return 0;
-}
+  Track *track1;
+  Track *track2;
+  gint column;
+  GtkSortType order;
+  gint result;
 
+  gtk_tree_model_get (model, a, READOUT_COL, &track1, -1);
+  gtk_tree_model_get (model, b, READOUT_COL, &track2, -1);
+  if(gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE (model),
+					   &column, &order) == FALSE)
+      return 0;
+
+  result = tm_data_compare (track1, track2, column);
+  return result;
+}
 
 
 /* set/read the counter used to remember how often the sort column has
@@ -966,24 +1213,62 @@ gint default_comp  (GtkTreeModel *model,
 static gint tm_sort_counter (gint inc)
 {
     static gint cnt = 0;
-    if (inc <0) cnt = 0;
-    else        cnt += inc;
+
+    if (inc <0)
+    {
+	cnt = 0;
+    }
+    else
+    {
+	cnt += inc;
+    }   
     return cnt;
 }
 
+
+/* Redisplays the tracks in the track view according to the order
+ * stored in the sort tab view. This only works if the track view is
+ * not sorted --> skip if sorted */
+
+void tm_adopt_order_in_sorttab (void)
+{
+    if (prefs_get_tm_sort () == SORT_NONE)
+    {
+	GList *gl, *tracks = NULL;
+
+	/* retrieve the currently displayed tracks (non ordered) from
+	   the last sort tab or from the selected playlist if no sort
+	   tabs are being used */
+	tm_remove_all_tracks ();
+	tracks = display_get_selected_members (prefs_get_sort_tab_num()-1);
+	for (gl=tracks; gl; gl=gl->next)
+	    tm_add_track_to_track_model ((Track *)gl->data, NULL);
+    }
+}
+
+
 /* redisplay the contents of the track view in it's unsorted order */
 static void tm_unsort (void)
 {
-    GList *gl, *tracks = NULL;
+    if (track_treeview)
+    {
+	GtkTreeModel *model= gtk_tree_view_get_model (track_treeview);
 
-    /* retrieve the currently displayed tracks (non ordered) from the
-       last sort tab or from the selected playlist if no sort tabs are
-       being used */
-    tracks = display_get_selected_members (prefs_get_sort_tab_num() - 1);
-    tm_remove_all_tracks (TRUE);
-    for (gl=tracks; gl; gl=gl->next)
-	tm_add_track_to_track_model ((Track *)gl->data, NULL);
-    tm_sort_counter (-1);
+	prefs_set_tm_sort (SORT_NONE);
+	if (!BROKEN_GTK_TREE_SORT)
+	{
+	    gtk_tree_sortable_set_sort_column_id
+		(GTK_TREE_SORTABLE (model),
+		 GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
+		 GTK_SORT_ASCENDING);
+	    tm_adopt_order_in_sorttab ();
+	}
+	else
+	{
+	    gtkpod_warning (_("Cannot unsort track view because of a bug in the GTK lib you are using (%d.%d.%d < 2.5.4). Once you sort the track view, you cannot go back to the unsorted state.\n\n"), gtk_major_version, gtk_minor_version, gtk_micro_version);
+	}
+	tm_sort_counter (-1);
+    }
 }
 
 
@@ -1000,10 +1285,9 @@ tm_track_column_button_clicked(GtkTreeViewColumn *tvc, gpointer data)
 	tm_sort_counter (-1);
 	lastcol = newcol;
     }
-    
+
     if (tm_sort_counter (1) >= 3)
     { /* after clicking three times, reset sort order! */
-	prefs_set_tm_sort (SORT_NONE);
 	tm_unsort ();  /* also resets sort counter */
     }
     else
@@ -1012,6 +1296,7 @@ tm_track_column_button_clicked(GtkTreeViewColumn *tvc, gpointer data)
     }
     prefs_set_tm_sortcol (newcol);
     if(prefs_get_tm_autostore ())  tm_rows_reordered ();
+/*     sort_window_update (); */
 }
 
 
@@ -1031,7 +1316,13 @@ void tm_sort (TM_item col, GtkSortType order)
 	    GtkSortType order;
 	    if (gtk_tree_sortable_get_sort_column_id
 		(GTK_TREE_SORTABLE (model), &column, &order))
-		tm_unsort ();
+	    {
+		/* column == -2 actually is not defined, but it means
+		   that the model is unsorted. The sortable interface
+		   is badly implemented in gtk 2.4 */
+		if (column != -2)
+		    tm_unsort ();
+	    }
 	}
     }
 }
@@ -1056,12 +1347,14 @@ static GtkTreeViewColumn *tm_add_text_column (TM_item col_id,
     }
     g_object_set_data (G_OBJECT (renderer), "editable", (gint *)editable);
     g_object_set_data (G_OBJECT (renderer), "column", (gint *)col_id);
-    column = gtk_tree_view_column_new_with_attributes (name, renderer, NULL);
+    column = gtk_tree_view_column_new ();
+    gtk_tree_view_column_set_title (column, name);
+    gtk_tree_view_column_pack_end (column, renderer, FALSE);
     gtk_tree_view_column_set_cell_data_func (column, renderer,
 					     tm_cell_data_func, NULL, NULL);
     gtk_tree_view_column_set_sort_column_id (column, col_id);
     gtk_tree_view_column_set_resizable (column, TRUE);
-    gtk_tree_view_column_set_clickable(column, TRUE);
+/*     gtk_tree_view_column_set_clickable(column, TRUE); */
     gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_column_set_fixed_width (column,
 					  prefs_get_tm_col_width (col_id));
@@ -1081,10 +1374,13 @@ static GtkTreeViewColumn *tm_add_text_column (TM_item col_id,
 /* Adds the columns to our track_treeview */
 static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
 {
+  GtkTreeModel *model = gtk_tree_view_get_model (track_treeview);
   GtkTreeViewColumn *col = NULL;
   gchar *text = NULL;
   gboolean editable = TRUE;          /* default */
   GtkCellRenderer *renderer = NULL;  /* default */
+  GtkTooltips *tt = GTK_TOOLTIPS (lookup_widget (gtkpod_window,
+						 "tooltips"));
 
   if ((tm_item) < 0 || (tm_item >= TM_NUM_COLUMNS))  return NULL;
 
@@ -1097,8 +1393,12 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
   case TM_COLUMN_ALBUM:
   case TM_COLUMN_GENRE:
   case TM_COLUMN_COMPOSER:
-  case TM_COLUMN_VOLUME:
+  case TM_COLUMN_FDESC:
+  case TM_COLUMN_GROUPING:
   case TM_COLUMN_RATING:
+  case TM_COLUMN_BITRATE:
+  case TM_COLUMN_SAMPLERATE:
+  case TM_COLUMN_BPM:
       break;
   case TM_COLUMN_TRACK_NR:
       text = _("#");
@@ -1111,6 +1411,7 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
       editable = FALSE;
       break;
   case TM_COLUMN_PC_PATH:
+  case TM_COLUMN_IPOD_PATH:
       editable = FALSE;
       break;
   case TM_COLUMN_TRANSFERRED:
@@ -1118,6 +1419,11 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
       editable = FALSE;
       renderer = gtk_cell_renderer_toggle_new ();
       break;
+  case TM_COLUMN_COMPILATION:
+      text = _("Cmpl");
+      editable = FALSE;
+      renderer = gtk_cell_renderer_toggle_new ();
+      break;
   case TM_COLUMN_SIZE:
       editable = FALSE;
       break;
@@ -1125,30 +1431,55 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
       text = _("Time");
       editable = FALSE;
       break;
-  case TM_COLUMN_BITRATE:
-      editable = FALSE;
-      break;
   case TM_COLUMN_PLAYCOUNT:
       text = _("Plycnt");
       break;
   case TM_COLUMN_TIME_PLAYED:
       text = _("Played");
-      editable = FALSE;
       break;
   case TM_COLUMN_TIME_MODIFIED:
       text = _("Modified");
-      editable = FALSE;
+      break;
+  case TM_COLUMN_TIME_CREATED:
+      text = _("Created");
       break;
   case TM_COLUMN_YEAR:
       text = _("Year");
       break;
+  case TM_COLUMN_VOLUME:
+      text = _("Vol.");
+      break;
+  case TM_COLUMN_SOUNDCHECK:
+      text = _("Sndchk.");
+      break;
   case TM_NUM_COLUMNS:
       break;
   }
   col = tm_add_text_column (tm_item, text, renderer, editable, pos);
+  if (col && (tm_item == TM_COLUMN_TITLE))
+  {
+      renderer = gtk_cell_renderer_toggle_new ();
+      g_object_set_data (G_OBJECT (renderer), "column",
+			 (gint *)tm_item);
+      g_signal_connect (G_OBJECT (renderer), "toggled",
+			G_CALLBACK (tm_cell_toggled), model);
+      gtk_tree_view_column_pack_start (col, renderer, FALSE);
+      gtk_tree_view_column_set_cell_data_func (col, renderer, tm_cell_data_func_toggle, NULL, NULL);
+  }
+  if (col && (tm_item == TM_COLUMN_COMPILATION))
+  {
+      g_signal_connect (G_OBJECT (renderer), "toggled",
+			G_CALLBACK (tm_cell_toggled), model);
+  }      
   if (col && (pos != -1))
+  {
       gtk_tree_view_column_set_visible (col,
 					prefs_get_col_visible (tm_item));
+  }
+  if (tt && tm_col_tooltips[tm_item])
+      gtk_tooltips_set_tip (tt, col->button, 
+			    gettext (tm_col_tooltips[tm_item]),
+			    NULL);
   return col;
 }
 
@@ -1196,7 +1527,7 @@ tm_button_press_event(GtkWidget *w, GdkEventButton *e, gpointer data)
 	    default:
 		break;
 	}
-	
+
     }
     return(FALSE);
 }
@@ -1243,7 +1574,7 @@ void tm_create_treeview (void)
 		       tm_drag_types, TGNR (tm_drag_types),
 		       GDK_ACTION_COPY|GDK_ACTION_MOVE);
   gtk_tree_view_enable_model_drag_dest(track_treeview, tm_drop_types,
-				       TGNR (tm_drop_types), 
+				       TGNR (tm_drop_types),
 				       GDK_ACTION_COPY|GDK_ACTION_MOVE);
   /* need the gtk_drag_dest_set() with no actions ("0") so that the
      data_received callback gets the correct info value. This is most
@@ -1273,7 +1604,7 @@ tm_show_preferred_columns(void)
     GtkTreeViewColumn *tvc = NULL;
     gboolean visible;
     gint i;
-    
+
     for (i=0; i<TM_NUM_COLUMNS; ++i)
     {
 	tvc = gtk_tree_view_get_column (track_treeview, i);
@@ -1304,6 +1635,81 @@ void tm_update_default_sizes (void)
 }
 
 
+/* Compare function to avoid sorting */
+static gint tm_nosort_comp (GtkTreeModel *model,
+			    GtkTreeIter *a,
+			    GtkTreeIter *b,
+			    gpointer user_data)
+{
+    return 0;
+}
+
+
+/* Disable sorting of the view during lengthy updates. */
+/* @enable: TRUE: enable, FALSE: disable */
+void tm_enable_disable_view_sort (gboolean enable)
+{
+    static gint disable_count = 0;
+
+    if (enable)
+    {
+	disable_count--;
+	if (disable_count < 0)
+	    fprintf (stderr, "Programming error: disable_count < 0\n");
+	if (disable_count == 0 && track_treeview)
+	{
+	    if ((prefs_get_tm_sort() != SORT_NONE) &&
+		prefs_get_disable_sorting ())
+	    {
+		/* Re-enable sorting */
+		GtkTreeModel *model = gtk_tree_view_get_model (track_treeview);
+		if (BROKEN_GTK_TREE_SORT)
+		{
+		    gtk_tree_sortable_set_sort_func (
+			GTK_TREE_SORTABLE (model),
+			prefs_get_tm_sortcol (),
+			tm_data_compare_func, NULL, NULL);
+		}
+		else
+		{
+		    gtk_tree_sortable_set_sort_column_id (
+			GTK_TREE_SORTABLE (model),
+			prefs_get_tm_sortcol (),
+			prefs_get_tm_sort ());
+		}
+	    }
+	}
+    }
+    else
+    {
+	if (disable_count == 0 && track_treeview)
+	{
+	    if ((prefs_get_tm_sort() != SORT_NONE) &&
+		prefs_get_disable_sorting ())
+	    {
+		/* Disable sorting */
+		GtkTreeModel *model = gtk_tree_view_get_model (track_treeview);
+		if (BROKEN_GTK_TREE_SORT)
+		{
+		    gtk_tree_sortable_set_sort_func (
+			GTK_TREE_SORTABLE (model),
+			prefs_get_tm_sortcol (),
+			tm_nosort_comp, NULL, NULL);
+		}
+		else
+		{
+		    gtk_tree_sortable_set_sort_column_id (
+			GTK_TREE_SORTABLE (model),
+			GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
+			prefs_get_tm_sort ());
+		}
+	    }
+	}
+	disable_count++;
+    }
+}
+
+
 
 
 /* ---------------------------------------------------------------- */
@@ -1407,7 +1813,7 @@ void tm_addtrackfunc (Playlist *plitem, Track *track, gpointer data)
     case GTK_TREE_VIEW_DROP_BEFORE:
 	gtk_list_store_insert_before (GTK_LIST_STORE (model),
 				      &new_iter, asf->to_iter);
-	break;	
+	break;
     }
     /* set the iter */
     tm_add_track_to_track_model (track, &new_iter);
@@ -1482,14 +1888,14 @@ gboolean tm_add_filelist (gchar *data,
 /*
  * utility function for appending ipod track ids for track view (DND)
  */
-void 
-on_tm_dnd_get_id_foreach(GtkTreeModel *tm, GtkTreePath *tp, 
+void
+on_tm_dnd_get_id_foreach(GtkTreeModel *tm, GtkTreePath *tp,
 			 GtkTreeIter *i, gpointer data)
 {
     Track *s;
     GString *filelist = (GString *)data;
 
-    gtk_tree_model_get(tm, i, READOUT_COL, &s, -1); 
+    gtk_tree_model_get(tm, i, READOUT_COL, &s, -1);
     if(s)
     {
 	g_string_append_printf (filelist, "%d\n", s->ipod_id);
@@ -1500,8 +1906,8 @@ on_tm_dnd_get_id_foreach(GtkTreeModel *tm, GtkTreePath *tp,
 /*
  * utility function for appending path for track view or playlist view (DND)
  */
-void 
-on_dnd_get_path_foreach(GtkTreeModel *tm, GtkTreePath *tp, 
+void
+on_dnd_get_path_foreach(GtkTreeModel *tm, GtkTreePath *tp,
 			GtkTreeIter *iter, gpointer data)
 {
     GString *filelist = (GString *)data;
@@ -1513,16 +1919,16 @@ on_dnd_get_path_foreach(GtkTreeModel *tm, GtkTreePath *tp,
 /*
  * utility function for appending file for track view (DND)
  */
-void 
-on_tm_dnd_get_file_foreach(GtkTreeModel *tm, GtkTreePath *tp, 
+void
+on_tm_dnd_get_file_foreach(GtkTreeModel *tm, GtkTreePath *tp,
 			   GtkTreeIter *iter, gpointer data)
 {
-    Track *s;
+    Track *track;
     GString *filelist = (GString *)data;
     gchar *name;
 
-    gtk_tree_model_get(tm, iter, READOUT_COL, &s, -1); 
-    name = get_track_name_on_disk_verified (s);
+    gtk_tree_model_get(tm, iter, READOUT_COL, &track, -1);
+    name = get_track_name_on_disk_verified (track);
     if (name)
     {
 	g_string_append_printf (filelist, "file:%s\n", name);
