@@ -1,36 +1,38 @@
-/* Time-stamp: <2004-01-12 17:28:15 jcs>
+/* Time-stamp: <2004-03-24 22:26:08 JST jcs>
 |
 |  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
 |  Part of the gtkpod project.
-| 
+|
 |  URL:  http://gtkpod.sourceforge.net/
-| 
+|
 |  This program is free software; you can redistribute it and/or modify
 |  it under the terms of the GNU General Public License as published by
 |  the Free Software Foundation; either version 2 of the License, or
 |  (at your option) any later version.
-| 
+|
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 |  GNU General Public License for more details.
-| 
+|
 |  You should have received a copy of the GNU General Public License
 |  along with this program; if not, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-| 
+|
 |  iTunes and iPod are trademarks of Apple
-| 
+|
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: mp3file.h,v 1.13 2004/01/17 09:37:49 jcsjcs Exp $
+|  $Id: mp3file.h,v 1.21 2004/03/24 13:27:04 jcsjcs Exp $
 */
 
 #ifndef MP3FILEH_INCLUDED
 #define MP3FILEH_INCLUDED 1
 
-#include "song.h"
+#include "track.h"
+
+gboolean mp3_write_file_info (gchar *filename, Track *track);
+Track *mp3_get_file_info (gchar *name);
+gboolean mp3_get_gain (gchar *path, Track *track);
 
-gboolean file_write_mp3_info (gchar *filename, Track *track);
-Track *file_get_mp3_info (gchar *name);
 #endif
