@@ -1,40 +1,41 @@
-/* Time-stamp: <2004-01-17 17:21:43 jcs>
+/* Time-stamp: <2004-12-29 18:13:35 jcs>
 |
 |  Copyright (C) 2002 Corey Donohoe <atmos at atmos.org>
 |  Part of the gtkpod project.
-| 
+|
 |  URL: http://gtkpod.sourceforge.net/
-| 
+|
 |  This program is free software; you can redistribute it and/or modify
 |  it under the terms of the GNU General Public License as published by
 |  the Free Software Foundation; either version 2 of the License, or
 |  (at your option) any later version.
-| 
+|
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 |  GNU General Public License for more details.
-| 
+|
 |  You should have received a copy of the GNU General Public License
 |  along with this program; if not, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-| 
+|
 |  iTunes and iPod are trademarks of Apple
-| 
+|
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: prefs_window.c,v 1.62 2004/01/17 09:37:49 jcsjcs Exp $
+|  $Id: prefs_window.c,v 1.86 2004/12/29 10:00:32 jcsjcs Exp $
 */
 
 #include <stdio.h>
 #include <string.h>
+#include "callbacks.h"
+#include "charset.h"
+#include "interface.h"
+#include "misc.h"
 #include "prefs.h"
 #include "prefs_window.h"
-#include "song.h"
-#include "interface.h"
-#include "callbacks.h"
 #include "support.h"
-#include "charset.h"
+#include "track.h"
 
 static GtkWidget *prefs_window = NULL;
 static GtkWidget *sort_window = NULL;
@@ -49,6 +50,66 @@ static GtkWidget *autoselect_widget[SORT_TAB_MAX];
 static void prefs_window_set_st_autoselect (guint32 inst, gboolean autoselect);
 static void prefs_window_set_autosettags (gint category, gboolean autoset);
 static void prefs_window_set_col_visible (gint column, gboolean visible);
+static void prefs_window_set_path (PathType i, const gchar *path);
+
+/* Definition of path button names.
+   E.g. path_button_names[PATH_PLAY_ENQUEUE] is
+   "play_enqueue_path_button".
+   path_fileselector_titles[] specifies the title for the file
+   chooser. path_type[] specifies whether to browse for dirs or for
+   files.
+*/
+static const gchar *path_button_names[] =
+{
+    "play_now_path_button",
+    "play_enqueue_path_button",
+    "mp3gain_path_button",
+    "sync_contacts_path_button",
+    "sync_calendar_path_button",
+    "mserv_music_root_button",
+    "mserv_trackinfo_root_button",
+    "sync_notes_path_button",
+    NULL
+};
+const gchar *path_entry_names[] =
+{
+    "play_now_path_entry",
+    "play_enqueue_path_entry",
+    "mp3gain_path_entry",
+    "sync_contacts_path_entry",
+    "sync_calendar_path_entry",
+    "mserv_music_root_entry",
+    "mserv_trackinfo_root_entry",
+    "sync_notes_path_entry",
+    NULL
+};
+static const gchar *path_fileselector_titles[] =
+{
+    N_("Please select command for 'Play Now'"),
+    N_("Please select command for 'Enqueue'"),
+    N_("Please select the mp3gain executable"),
+    N_("Please select command to sync contacts"),
+    N_("Please select command to sync calendar"),
+    N_("Select the mserv music root directory"),
+    N_("Select the mserv trackinfo root directory"),
+    NULL
+};
+static const GtkFileChooserAction path_type[] =
+{
+    GTK_FILE_CHOOSER_ACTION_OPEN,  /* select file */
+    GTK_FILE_CHOOSER_ACTION_OPEN,
+    GTK_FILE_CHOOSER_ACTION_OPEN,
+    GTK_FILE_CHOOSER_ACTION_OPEN,
+    GTK_FILE_CHOOSER_ACTION_OPEN,
+    GTK_FILE_CHOOSER_ACTION_OPEN,
+    GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, /* select folder */
+    GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+    -1
+};
+
+/* pointers to fileselectors */
+static GtkWidget *path_filechooser[PATH_NUM];
+
 
 static void on_cfg_st_autoselect_toggled (GtkToggleButton *togglebutton,
 					  gpointer         user_data)
@@ -76,6 +137,154 @@ static void on_cfg_col_visible_toggled (GtkToggleButton *togglebutton,
 }
 
 
+/* Close all open path filechoosers */
+static void path_close_windows (void)
+{
+    gint i;
+    for (i=0; i<PATH_NUM; ++i)
+    {
+	if (path_filechooser[i])
+	{
+	    gtk_widget_destroy (path_filechooser[i]);
+	    path_filechooser[i] = NULL;
+	}
+    }
+}
+
+
+/* ok button on path fileselector window was pressed */
+static void on_path_ok (PathType i)
+{
+    g_return_if_fail (i>=0 && i<PATH_NUM);
+
+    if (path_filechooser[i])
+    {
+	const gchar *npath = gtk_file_chooser_get_filename (
+	    GTK_FILE_CHOOSER (path_filechooser[i]));
+	const gchar *opath = tmpcfg->path[i];
+	/* find command line arguments */
+	const gchar *opathp = strchr (opath, ' ');
+	gchar *newpath;
+	GtkWidget *w;
+
+	/* attach command line arguments if present */
+	if (opathp)
+	    newpath = g_strdup_printf ("%s%s", npath, opathp);
+	else
+	    newpath = g_strdup (npath);
+
+	if ((w = lookup_widget (prefs_window, path_entry_names[i])))
+	{
+	    gchar *newpath_utf8 = g_filename_to_utf8 (newpath, -1, NULL, NULL, NULL);
+	    gtk_entry_set_text(GTK_ENTRY(w), newpath_utf8);
+	    g_free (newpath_utf8);
+	}
+	g_free (newpath);
+
+	gtk_widget_destroy (path_filechooser[i]);
+	path_filechooser[i] = NULL;
+    }
+}
+
+
+static void on_fc_response (GtkFileChooser *fc,
+			    gint response,
+			    PathType i)
+{
+    g_return_if_fail (i>=0 && i<PATH_NUM && path_filechooser[i]);
+
+    switch (response)
+    {
+    case GTK_RESPONSE_CANCEL:
+	gtk_widget_destroy (path_filechooser[i]);
+	path_filechooser[i] = NULL;
+	break;
+    case GTK_RESPONSE_ACCEPT:
+	on_path_ok (i);
+	break;
+    case GTK_RESPONSE_NONE:
+    case GTK_RESPONSE_DELETE_EVENT:
+	path_filechooser[i] = NULL;
+	break;
+    default:
+	printf ("programming error: received unknown response %d in on_fc_response\n", response);
+	break;
+    }
+}
+
+
+/* one of the "..." buttons has been pressed -> open a file chooser
+   dialog and let the user select a file or directory */
+static void on_path_button_pressed (GtkButton *button, gpointer user_data)
+{
+    PathType i = (PathType)user_data;
+    GtkFileChooser *fc;
+    const gchar *path, *pathp;
+    gchar *buf, *fbuf;
+
+    g_return_if_fail (tmpcfg);
+    g_return_if_fail (i>=0 && i<PATH_NUM);
+
+    if (path_filechooser[i])
+    {  /* filechooser already open --> simply raise to the top */
+	gdk_window_raise(path_filechooser[i]->window);
+	return;
+    }
+    path_filechooser[i] = gtk_file_chooser_dialog_new (
+	_(path_fileselector_titles[i]),
+	NULL,
+	path_type[i],
+	GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+	NULL);
+    /* for convenience */
+    fc = GTK_FILE_CHOOSER (path_filechooser[i]);
+    g_signal_connect (fc, "response",
+		      G_CALLBACK (on_fc_response),
+		      (gpointer)i);
+    /* find first whitespace separating path from command line
+     * arguments */
+    path = tmpcfg->path[i];
+    pathp = strchr (path, ' ');
+    if (pathp)
+	buf = g_strndup (path, pathp-path);
+    else
+	buf = g_strdup (path);
+
+    /* get full path */
+    fbuf = g_find_program_in_path (buf);
+    if (!fbuf) fbuf = g_strdup (buf);
+
+    if (fbuf)
+    {
+	gchar *fbuf_utf8 = g_filename_from_utf8 (fbuf, -1, NULL, NULL, NULL);
+	if (path_type[i] == GTK_FILE_CHOOSER_ACTION_OPEN)
+	{
+	    gtk_file_chooser_set_filename (fc, fbuf_utf8);
+	}
+	if (path_type[i] == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)
+	{
+	    gtk_file_chooser_set_current_folder (fc, fbuf_utf8);
+	}
+	g_free (fbuf_utf8);
+	g_free (fbuf);
+    }
+    g_free (buf);
+
+    gtk_widget_show (path_filechooser[i]);
+}
+
+
+static void on_path_entry_changed (GtkEditable     *editable,
+				   gpointer         user_data)
+{
+    PathType i = (PathType)user_data;
+    gchar *buf = gtk_editable_get_chars(editable, 0, -1);
+    prefs_window_set_path (i, buf);
+    g_free (buf);
+}
+
+
 /* turn the prefs window insensitive (if it's open) */
 void prefs_window_block (void)
 {
@@ -121,7 +330,8 @@ prefs_window_create(void)
     if(!prefs_window)
     {
 	GtkWidget *w = NULL;
-	
+	GtkTooltips *tt = NULL;
+
 	if(!tmpcfg && !origcfg)
 	{
 	    tmpcfg = clone_prefs();
@@ -146,61 +356,6 @@ prefs_window_create(void)
 		g_free (buf);
 	    }
 	}
-	if((w = lookup_widget(prefs_window, "cfg_filename_format")))
-	{
-	    if (tmpcfg->filename_format)
-	    {  /* we should copy the new path first because by setting
-		  the text we might get a callback destroying the old
-		  value... */
-		gchar *buf = g_strdup (tmpcfg->filename_format);
-		gtk_entry_set_text(GTK_ENTRY(w), buf);
-		g_free (buf);
-	    }
-	}
-	if((w = lookup_widget(prefs_window, "play_now_path_entry")))
-	{
-	    if (tmpcfg->play_now_path)
-	    {  /* we should copy the new path first because by setting
-		  the text we might get a callback destroying the old
-		  value... */
-		gchar *buf = g_strdup (tmpcfg->play_now_path);
-		gtk_entry_set_text(GTK_ENTRY(w), buf);
-		g_free (buf);
-	    }
-	}
-	if((w = lookup_widget(prefs_window, "play_enqueue_path_entry")))
-	{
-	    if (tmpcfg->play_enqueue_path)
-	    {  /* we should copy the new path first because by setting
-		  the text we might get a callback destroying the old
-		  value... */
-		gchar *buf = g_strdup (tmpcfg->play_enqueue_path);
-		gtk_entry_set_text(GTK_ENTRY(w), buf);
-		g_free (buf);
-	    }
-	}
-	if((w = lookup_widget(prefs_window, "mp3gain_path_entry")))
-	{
-	    if (tmpcfg->mp3gain_path)
-	    {  /* we should copy the new path first because by setting
-		  the text we might get a callback destroying the old
-		  value... */
-		gchar *buf = g_strdup (tmpcfg->mp3gain_path);
-		gtk_entry_set_text(GTK_ENTRY(w), buf);
-		g_free (buf);
-	    }
-	}
-	if((w = lookup_widget(prefs_window, "time_format_entry")))
-	{
-	    if (tmpcfg->time_format)
-	    {  /* we should copy the new path first because by setting
-		  the text we might get a callback destroying the old
-		  value... */
-		gchar *buf = g_strdup (tmpcfg->time_format);
-		gtk_entry_set_text(GTK_ENTRY(w), buf);
-		g_free (buf);
-	    }
-	}
 	if((w = lookup_widget(prefs_window, "charset_combo")))
 	{
 	    charset_init_combo (GTK_COMBO (w));
@@ -382,6 +537,32 @@ prefs_window_create(void)
 		g_free (buf);
 	    }
 	}
+	/* connect signals for path entrys and selectors */
+	for (i=0; i<PATH_NUM; ++i)
+	{
+	    if ((w = lookup_widget (prefs_window, path_button_names[i])))
+	    {
+		g_signal_connect ((gpointer)w,
+				  "clicked",
+				  G_CALLBACK (on_path_button_pressed),
+				  (gpointer)i);
+	    }
+	    if ((w = lookup_widget (prefs_window, path_entry_names[i])))
+	    {
+		if (tmpcfg->path[i])
+		{  /* we should copy the new path first because by setting
+		      the text we might get a callback destroying the old
+		      value... */
+		    gchar *buf = g_strdup (tmpcfg->path[i]);
+		    gtk_entry_set_text(GTK_ENTRY(w), buf);
+		    g_free (buf);
+		}
+		g_signal_connect ((gpointer)w,
+				  "changed",
+				  G_CALLBACK (on_path_entry_changed),
+				  (gpointer)i);
+	    }
+	}
 	if((w = lookup_widget(prefs_window, "cfg_mpl_autoselect")))
 	{
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
@@ -433,10 +614,11 @@ prefs_window_create(void)
 	    }
 	    gtk_widget_set_sensitive (w, tmpcfg->parsetags);
 	}
+	/* get tooltips */
+	tt = GTK_TOOLTIPS (lookup_widget (prefs_window, "tooltips"));
 	for (i=0; i<TM_NUM_COLUMNS; ++i)
 	{
-	    gchar *buf;
-	    buf = g_strdup_printf ("col_visible%d", i);
+	    gchar *buf = g_strdup_printf ("col_visible%d", i);
 	    if((w = lookup_widget(prefs_window,  buf)))
 	    {
 		/* set label */
@@ -444,9 +626,16 @@ prefs_window_create(void)
 				      gettext (tm_col_strings[i]));
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 					     tmpcfg->col_visible[i]);
+		/* set tooltip if available */
+		if (tt && tm_col_tooltips[i])
+		{
+		    gtk_tooltips_set_tip (tt, w, 
+					  gettext (tm_col_tooltips[i]),
+					  NULL);
+		}
 		/* glade makes a "GTK_OBJECT (i)" which segfaults
-		   because "i" is not a GTK object. So we have to set up
-		   the signal handlers ourselves */
+		   because "i" is not a GTK object. So we have to set
+		   up the signal handlers ourselves */
 		g_signal_connect ((gpointer)w,
 				  "toggled",
 				  G_CALLBACK (on_cfg_col_visible_toggled),
@@ -454,12 +643,7 @@ prefs_window_create(void)
 	    }
 	    g_free (buf);
 	}
-	
-	if((w = lookup_widget(prefs_window, "cfg_keep_backups")))
-	{
-	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
-					    tmpcfg->keep_backups);
-	}
+
 	if((w = lookup_widget(prefs_window, "cfg_write_extended")))
 	{
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
@@ -484,16 +668,53 @@ prefs_window_create(void)
 				       prefs_get_sort_tab_num ());
 	    prefs_window_set_sort_tab_num (tmpcfg->sort_tab_num);
 	}
-	if((w = lookup_widget(prefs_window, "cfg_write_gaintag")))
+	if((w = lookup_widget(prefs_window, "concal_autosync")))
 	{
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
-					 tmpcfg->write_gaintag);
-        }
-	if((w = lookup_widget(prefs_window, "cfg_special_export_charset")))
+					 tmpcfg->concal_autosync);
+	}
+	if((w = lookup_widget(prefs_window, "cfg_tmp_disable_sort")))
 	{
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
-					 tmpcfg->special_export_charset);
-        }
+					 tmpcfg->tmp_disable_sort);
+	}
+	if((w = lookup_widget(prefs_window, "cfg_startup_messages")))
+	{
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
+					 tmpcfg->startup_messages);
+	}
+	if((w = lookup_widget(prefs_window, "cfg_mserv_use")))
+	{
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
+					 tmpcfg->mserv_use);
+	}
+	if((w = lookup_widget(prefs_window, "cfg_mserv_report_probs")))
+	{
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
+					 tmpcfg->mserv_report_probs);
+	}
+	if((w = lookup_widget(prefs_window, "mserv_username_entry")))
+	{
+	    if (tmpcfg->mserv_username)
+	    {  /* we should copy the new path first because by setting
+		  the text we might get a callback destroying the old
+		  value... */
+		gchar *buf = g_strdup (tmpcfg->mserv_username);
+		gtk_entry_set_text(GTK_ENTRY(w), buf);
+		g_free (buf);
+	    }
+	}
+/* 	if((w = lookup_widget(prefs_window, "cfg_unused_gboolean3"))) */
+/* 	{ */
+/* 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), */
+/* 					 tmpcfg->unused_gboolean3); */
+/* 	} */
+	if((w = lookup_widget(prefs_window, "concal_label")))
+	{
+	    gchar *str = g_strdup_printf (_("Have a look at the scripts provided in '%s'. If you write a new script, please send it to jcsjcs at users.sourceforge.net for inclusion into the next release."), PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "scripts" G_DIR_SEPARATOR_S);
+	    gtk_label_set_text (GTK_LABEL (w), str);
+	    g_free (str);
+	}
 	prefs_window_show_hide_tooltips ();
 	gtk_widget_show(prefs_window);
     }
@@ -505,7 +726,7 @@ prefs_window_create(void)
  * UI has requested preferences update(by clicking ok on the prefs window)
  * Frees the tmpcfg variable
  */
-static void 
+static void
 prefs_window_set(void)
 {
     if (tmpcfg)
@@ -516,13 +737,13 @@ prefs_window_set(void)
 	tm_store_col_order ();
 	prefs_set_id3_write(tmpcfg->id3_write);
 	prefs_set_id3_write_id3v24(tmpcfg->id3_write_id3v24);
-	prefs_set_mount_point(tmpcfg->ipod_mount);
-	prefs_set_play_now_path(tmpcfg->play_now_path);
-	prefs_set_play_enqueue_path(tmpcfg->play_enqueue_path);
-	prefs_set_mp3gain_path(tmpcfg->mp3gain_path);
-	prefs_set_time_format(tmpcfg->time_format);
+	prefs_set_ipod_mount(tmpcfg->ipod_mount);
+	for (i=0; i<PATH_NUM; ++i)
+	{
+	    prefs_set_path (i, tmpcfg->path[i]);
+	}
 	prefs_set_charset(tmpcfg->charset);
-	prefs_set_auto_import(tmpcfg->autoimport);
+	prefs_set_autoimport(tmpcfg->autoimport);
 	for (i=0; i<SORT_TAB_MAX; ++i) {
 	    prefs_set_st_autoselect (i, tmpcfg->st[i].autoselect);
 	    prefs_set_st_category (i, tmpcfg->st[i].category);
@@ -543,7 +764,10 @@ prefs_window_set(void)
 	prefs_set_track_ipod_file_deletion(tmpcfg->deletion.ipod_file);
 	prefs_set_sync_remove_confirm(tmpcfg->deletion.syncing);
 	prefs_set_write_extended_info(tmpcfg->write_extended_info);
-	prefs_set_keep_backups(tmpcfg->keep_backups);
+	prefs_set_show_duplicates(tmpcfg->show_duplicates);
+	prefs_set_show_updated(tmpcfg->show_updated);
+	prefs_set_show_non_updated(tmpcfg->show_non_updated);
+	prefs_set_show_sync_dirs(tmpcfg->show_sync_dirs);
 	/* we delete all stored md5 checksums if the md5 checksumming got
 	   disabled */
 	if (prefs_get_md5tracks() && !tmpcfg->md5tracks)
@@ -553,10 +777,6 @@ prefs_window_set(void)
 	prefs_set_update_existing(tmpcfg->update_existing);
 	prefs_set_block_display(tmpcfg->block_display);
 	prefs_set_sort_tab_num(tmpcfg->sort_tab_num, TRUE);
-	prefs_set_show_duplicates(tmpcfg->show_duplicates);
-	prefs_set_show_updated(tmpcfg->show_updated);
-	prefs_set_show_non_updated(tmpcfg->show_non_updated);
-	prefs_set_show_sync_dirs(tmpcfg->show_sync_dirs);
 	prefs_set_sync_remove(tmpcfg->sync_remove);
 	prefs_set_toolbar_style(tmpcfg->toolbar_style);
 	prefs_set_display_toolbar(tmpcfg->display_toolbar);
@@ -570,9 +790,13 @@ prefs_window_set(void)
 	prefs_set_write_charset(tmpcfg->write_charset);
 	prefs_set_add_recursively(tmpcfg->add_recursively);
 	prefs_set_automount(tmpcfg->automount);
-	prefs_set_filename_format(tmpcfg->filename_format);
-	prefs_set_write_gaintag(tmpcfg->write_gaintag);
-	prefs_set_special_export_charset(tmpcfg->special_export_charset);
+	prefs_set_concal_autosync(tmpcfg->concal_autosync);
+	prefs_set_tmp_disable_sort(tmpcfg->tmp_disable_sort);
+	prefs_set_startup_messages(tmpcfg->startup_messages);
+	prefs_set_mserv_use(tmpcfg->mserv_use);
+	prefs_set_mserv_report_probs(tmpcfg->mserv_report_probs);
+	prefs_set_mserv_username(tmpcfg->mserv_username);
+	prefs_set_unused_gboolean3(tmpcfg->unused_gboolean3);
 
 	tm_show_preferred_columns();
     }
@@ -619,6 +843,9 @@ prefs_window_cancel(void)
     if(prefs_window)
 	gtk_widget_destroy(prefs_window);
     prefs_window = NULL;
+
+    /* close path_fileselector windows */
+    path_close_windows ();
 }
 
 /* when window is deleted, we keep the currently applied prefs and
@@ -647,6 +874,9 @@ void prefs_window_delete(void)
     if(prefs_window)
 	gtk_widget_destroy(prefs_window);
     prefs_window = NULL;
+
+    /* close path_fileselector windows */
+    path_close_windows ();
 }
 
 /* apply the current settings and close the window */
@@ -679,6 +909,9 @@ prefs_window_ok (void)
     if(prefs_window)
 	gtk_widget_destroy(prefs_window);
     prefs_window = NULL;
+
+    /* close path_fileselector windows */
+    path_close_windows ();
 }
 
 
@@ -686,30 +919,25 @@ prefs_window_ok (void)
 void
 prefs_window_apply (void)
 {
-    gint defx, defy;
+    gint defx, defy, i;
     GtkWidget *nb, *w;
 
     /* save current settings */
     prefs_window_set ();
+
     /* reset the validated path entries */
-    if((w = lookup_widget(prefs_window, "play_now_path_entry")))
+    for (i=0; i<PATH_NUM; ++i)
     {
-	gtk_entry_set_text(GTK_ENTRY(w), prefs_get_play_now_path ());
-	/* tmpcfg gets set by the "changed" callback */
+	if((w = lookup_widget(prefs_window, path_entry_names[i])))
+	{
+	    gtk_entry_set_text(GTK_ENTRY(w), tmpcfg->path[i]);
+	    /* tmpcfg gets set by the "changed" callback */
+	}
     }
-    if((w = lookup_widget(prefs_window, "play_enqueue_path_entry")))
+
+    if((w = lookup_widget(prefs_window, "mserv_username_entry")))
     {
-	gtk_entry_set_text(GTK_ENTRY(w), prefs_get_play_enqueue_path ());
-	/* tmpcfg gets set by the "changed" callback */
-    }
-    if((w = lookup_widget(prefs_window, "mp3gain_path_entry")))
-    {
-	gtk_entry_set_text(GTK_ENTRY(w), prefs_get_mp3gain_path ());
-	/* tmpcfg gets set by the "changed" callback */
-    }
-    if((w = lookup_widget(prefs_window, "time_format_entry")))
-    {
-	gtk_entry_set_text(GTK_ENTRY(w), prefs_get_time_format ());
+	gtk_entry_set_text(GTK_ENTRY(w), prefs_get_mserv_username ());
 	/* tmpcfg gets set by the "changed" callback */
     }
 
@@ -727,7 +955,7 @@ prefs_window_apply (void)
 /**
  * prefs_window_set_md5tracks
  * @val - truth value of whether or not we should use the md5 hash to
- * prevent file duplication. changes temp variable 
+ * prevent file duplication. changes temp variable
  */
 void
 prefs_window_set_md5tracks(gboolean val)
@@ -785,63 +1013,39 @@ prefs_window_set_mount_point(const gchar *mp)
     tmpcfg->ipod_mount = g_strdup(mp);
 }
 
-void prefs_window_set_play_now_path(const gchar *path)
+static void prefs_window_set_path (PathType i, const gchar *path)
 {
     if (!path) return;
-    g_free (tmpcfg->play_now_path);
-    tmpcfg->play_now_path = g_strdup (path);
+    g_return_if_fail (i>=0 && i<PATH_NUM);
+    g_free (tmpcfg->path[i]);
+    tmpcfg->path[i] = g_strdup (path);
 }
 
-void prefs_window_set_play_enqueue_path(const gchar *path)
-{
-    if (!path) return;
-    g_free (tmpcfg->play_enqueue_path);
-    tmpcfg->play_enqueue_path = g_strdup (path);
-}
-
-void prefs_window_set_mp3gain_path(const gchar *path)
-{
-    if (!path) return;
-    g_free (tmpcfg->mp3gain_path);
-    tmpcfg->mp3gain_path = g_strdup (path);
-}
-
-void prefs_window_set_time_format(const gchar *format)
-{
-    if (!format) return;
-    g_free (tmpcfg->time_format);
-    tmpcfg->time_format = g_strdup (format);
-}
-
-void prefs_window_set_keep_backups(gboolean active)
-{
-  tmpcfg->keep_backups = active;
-}
 void prefs_window_set_write_extended_info(gboolean active)
 {
   tmpcfg->write_extended_info = active;
 }
 
-void 
+void
 prefs_window_set_delete_track_ipod(gboolean val)
 {
     tmpcfg->deletion.ipod_file = val;
 }
 
-void 
+void
 prefs_window_set_delete_track_playlist(gboolean val)
 {
     tmpcfg->deletion.track = val;
 }
 
-void 
+void
 prefs_window_set_sync_remove_confirm(gboolean val)
 {
     tmpcfg->deletion.syncing = val;
 }
 
 void
-prefs_window_set_auto_import(gboolean val)
+prefs_window_set_autoimport(gboolean val)
 {
     tmpcfg->autoimport = val;
 }
@@ -1026,25 +1230,47 @@ void prefs_window_set_automount(gboolean val)
 }
 
 void
-prefs_window_set_filename_format(const gchar *val)
+prefs_window_set_concal_autosync(gboolean val)
 {
-    g_free (tmpcfg->filename_format);
-    tmpcfg->filename_format = g_strdup(val);
+    tmpcfg->concal_autosync = val;
 }
 
 void
-prefs_window_set_write_gaintag(gboolean val)
+prefs_window_set_tmp_disable_sort(gboolean val)
 {
-    tmpcfg->write_gaintag = val;
+    tmpcfg->tmp_disable_sort = val;
 }
 
 void
-prefs_window_set_special_export_charset(gboolean val)
+prefs_window_set_startup_messages(gboolean val)
 {
-    tmpcfg->special_export_charset = val;
+    tmpcfg->startup_messages = val;
 }
 
+void
+prefs_window_set_mserv_use(gboolean val)
+{
+    tmpcfg->mserv_use = val;
+}
 
+void
+prefs_window_set_mserv_report_probs(gboolean val)
+{
+    tmpcfg->mserv_report_probs = val;
+}
+
+void prefs_window_set_mserv_username(const gchar *val)
+{
+    if (!val) return;
+    g_free (tmpcfg->mserv_username);
+    tmpcfg->mserv_username = g_strdup (val);
+}
+
+void
+prefs_window_set_unused_gboolean3(gboolean val)
+{
+    tmpcfg->unused_gboolean3 = val;
+}
 
 
 /* ------------------------------------------------------------ *\
@@ -1060,11 +1286,14 @@ prefs_window_set_special_export_charset(gboolean val)
  */
 void sort_window_create (void)
 {
-    if (!sort_window)
+    if (sort_window)
+    {  /* sort options already open --> simply raise to the top */
+	gdk_window_raise(sort_window->window);
+    }
+    else
     {
-	GtkWidget *w;
-	gchar *str;
 	GList *collist = NULL;
+	GtkWidget *w;
 	gint i;
 
 	if(!tmpsortcfg && !origsortcfg)
@@ -1079,7 +1308,50 @@ void sort_window_create (void)
 	}
 	sort_window = create_sort_window ();
 
-	w = NULL;
+	/* Set Sort-Column-Combo */
+	/* create the list in the order of the columns displayed */
+	tm_store_col_order ();
+	for (i=0; i<TM_NUM_COLUMNS; ++i)
+	{   /* first the visible columns */
+	    TM_item col = prefs_get_col_order (i);
+	    if (col != -1)
+	    {
+		if (prefs_get_col_visible (col))
+		    collist = g_list_append (collist,
+					     gettext (tm_col_strings[col]));
+	    }
+	}
+	for (i=0; i<TM_NUM_COLUMNS; ++i)
+	{   /* first the visible columns */
+	    TM_item col = prefs_get_col_order (i);
+	    if (col != -1)
+	    {
+		if (!prefs_get_col_visible (col))
+		    collist = g_list_append (collist,
+					     gettext (tm_col_strings[col]));
+	    }
+	}
+	w = lookup_widget (sort_window, "sort_combo");
+	gtk_combo_set_popdown_strings (GTK_COMBO (w), collist);
+	g_list_free (collist);
+	collist = NULL;
+
+	sort_window_update ();
+
+	sort_window_show_hide_tooltips ();
+	gtk_widget_show (sort_window);
+    }
+}
+
+
+
+/* Update sort_window's settings */
+void sort_window_update (void)
+{
+    if (sort_window && tmpsortcfg)
+    {
+	gchar *str;
+	GtkWidget *w = NULL;
 	switch (tmpsortcfg->pm_sort)
 	{
 	case SORT_ASCENDING:
@@ -1111,10 +1383,20 @@ void sort_window_create (void)
 	if (w)
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
 
-	w = lookup_widget (sort_window, "tm_none");
+	switch (tmpsortcfg->tm_sort)
+	{
+	case SORT_ASCENDING:
+	    w = lookup_widget (sort_window, "tm_ascend");
+	    break;
+	case SORT_DESCENDING:
+	    w = lookup_widget (sort_window, "tm_descend");
+	    break;
+	case SORT_NONE:
+	    w = lookup_widget (sort_window, "tm_none");
+	    break;
+	}
 	if (w)
-	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
-					 (tmpsortcfg->tm_sort == SORT_NONE));
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
 
 	w = lookup_widget (sort_window, "pm_autostore");
 	if (w)
@@ -1125,49 +1407,20 @@ void sort_window_create (void)
 	if (w)
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 					 tmpsortcfg->tm_autostore);
+
 	if((w = lookup_widget(sort_window, "cfg_case_sensitive")))
 	{
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 					 tmpsortcfg->case_sensitive);
 	}
-	/* Set Sort-Column-Combo */
-	/* create the list in the order of the columns displayed */
-	tm_store_col_order ();
-	for (i=0; i<TM_NUM_COLUMNS; ++i)
-	{   /* first the visible columns */
-	    TM_item col = prefs_get_col_order (i);
-	    if (col != -1)
-	    {
-		if (prefs_get_col_visible (col))
-		    collist = g_list_append (collist,
-					     gettext (tm_col_strings[col]));
-	    }
-	}
-	for (i=0; i<TM_NUM_COLUMNS; ++i)
-	{   /* first the visible columns */
-	    TM_item col = prefs_get_col_order (i);
-	    if (col != -1)
-	    {
-		if (!prefs_get_col_visible (col))
-		    collist = g_list_append (collist,
-					     gettext (tm_col_strings[col]));
-	    }
-	}
-	w = lookup_widget (sort_window, "sort_combo");
-	gtk_combo_set_popdown_strings (GTK_COMBO (w), collist);
-	g_list_free (collist);
-	collist = NULL;
-	/* set standard entry */
+	/* set standard entry in combo */
 	str = gettext (tm_col_strings[prefs_get_tm_sortcol ()]);
+	w = lookup_widget (sort_window, "sort_combo");
 	gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (w)->entry), str);
-
-	sort_window_show_hide_tooltips ();
-	gtk_widget_show (sort_window);
     }
 }
 
 
-
 /* turn the sort window insensitive (if it's open) */
 void sort_window_block (void)
 {
@@ -1215,7 +1468,7 @@ static TM_item sort_window_get_sort_col (void)
 	    if (strcmp (gettext (tm_col_strings[i]), str) == 0)  break;
     if ((i<0) || (i>= TM_NUM_COLUMNS))
     {
-	fprintf (stderr, 
+	fprintf (stderr,
 		 "Programming error: cal_get_category () -- item not found.\n");
 	/* set to something reasonable at least */
 	i = TM_COLUMN_TITLE;
